﻿/* 
    Copyright (c) 2010 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/
using System;
using System.IO;
using System.Threading;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace SilverlightMicrophoneSample
{
    public partial class MainPage : PhoneApplicationPage
    {
        private Microphone microphone = Microphone.Default;     // Объектное представление физического микрофона в устройстве
        private byte[] buffer;                                  // Динамический буфер для получения аудио данных с микрофона
        private MemoryStream stream = new MemoryStream();       // Сохраняет аудиоданные для последующего воспроизведения
        private SoundEffectInstance soundInstance;              // Использоан для воспроизведения аудио
        private bool soundIsPlaying = false;                    // Флаг для мониторинга состояния воспроизведения аудио

        // Изображения статуса
        private BitmapImage blankImage;
        private BitmapImage microphoneImage;
        private BitmapImage speakerImage;


        /// Конструктор

        public MainPage()
        {
            InitializeComponent();

            // Таймер моделирует игровой цикл XNA Framework 
            // (микрофон взят из XNA Framework). Мы также используем этот таймер для мониторинга 
            // состояния воспроизведения аудио.
            DispatcherTimer dt = new DispatcherTimer();
            dt.Interval = TimeSpan.FromMilliseconds(33);
            dt.Tick += new EventHandler(dt_Tick);
            dt.Start();

            // Обработчик события для получения аудиоданных из буфера
            microphone.BufferReady += new EventHandler<EventArgs>(microphone_BufferReady);

            blankImage = new BitmapImage(new Uri("Images/blank.png", UriKind.RelativeOrAbsolute));
            microphoneImage = new BitmapImage(new Uri("Images/microphone.png", UriKind.RelativeOrAbsolute));
            speakerImage = new BitmapImage(new Uri("Images/speaker.png", UriKind.RelativeOrAbsolute));
        }

        /// Обновляет XNA FrameworkDispatcher и проверяет воспроизводится ли звук
        /// Если воспроизведение звука было остановлено, то это обновляет UI.

        void dt_Tick(object sender, EventArgs e)
        {
            try { FrameworkDispatcher.Update(); }
            catch { }

            if (true == soundIsPlaying)
            {
                if (soundInstance.State != SoundState.Playing)
                {
                    // Audio has finished playing
                    soundIsPlaying = false;

                    // Update the UI to reflect that the 
                    // sound has stopped playing
                    SetButtonStates(true, true, false);
                    UserHelp.Text = "Воспроизведение\nили запись";
                    StatusImage.Source = blankImage;
                }
            }
        }

        void microphone_BufferReady(object sender, EventArgs e)
        {
            // Получаем аудиоданные
            microphone.GetData(buffer);

            // Сохраняем аудиоданные в потоке
            stream.Write(buffer, 0, buffer.Length);
        }

        private void recordButton_Click(object sender, EventArgs e)
        {
            // Получение аудиоданных порциями по 1/2 секунды
            microphone.BufferDuration = TimeSpan.FromMilliseconds(500);

            // Выделяем память для хранения аудиоданных
            buffer = new byte[microphone.GetSampleSizeInBytes(microphone.BufferDuration)];

            stream.SetLength(0);

            // Начинаем запись
            microphone.Start();

            SetButtonStates(false, false, true);
            UserHelp.Text = "Запись";
            StatusImage.Source = microphoneImage;
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            if (microphone.State == MicrophoneState.Started)
            {
                microphone.Stop();
            }
            else if (soundInstance.State == SoundState.Playing)
            {
                soundInstance.Stop();
            }

            SetButtonStates(true, true, false);
            UserHelp.Text = "Готов";
            StatusImage.Source = blankImage;
        }

        private void playButton_Click(object sender, EventArgs e)
        {
            if (stream.Length > 0)
            {
                SetButtonStates(false, false, true);
                UserHelp.Text = "Воспроизведение";
                StatusImage.Source = speakerImage;

                // Воспроизводим аудио в новом потоке
                Thread soundThread = new Thread(new ThreadStart(playSound));
                soundThread.Start();
            }
        }

        private void playSound()
        {
            SoundEffect sound = new SoundEffect(stream.ToArray(), microphone.SampleRate, AudioChannels.Mono);
            soundInstance = sound.CreateInstance();
            soundIsPlaying = true;
            soundInstance.Play();
        }

        private void SetButtonStates(bool recordEnabled, bool playEnabled, bool stopEnabled)
        {
            (ApplicationBar.Buttons[0] as ApplicationBarIconButton).IsEnabled = recordEnabled;
            (ApplicationBar.Buttons[1] as ApplicationBarIconButton).IsEnabled = playEnabled;
            (ApplicationBar.Buttons[2] as ApplicationBarIconButton).IsEnabled = stopEnabled;
        }
    }
}
